//Build next PB constraints based on the previous output
//(For prioritization of minimization criteria)

#include <iostream>
#include <fstream>
#include <cstdio>
#include <string>
#include <cstdlib>


using namespace std;

//argv[0]	COMMAND
//argv[2]	input file name: constraints file (minisat+ opb format) of previous criterion
//argv[1]	input file name: output of PB solvers to previous criterion
//argv[3]	output file name: the next constraints file to the PB-solver
//argv[4]	output file name: the newly added constraint (only one constraint)

int main( int argc, char *argv[] ){

  //check whether input files exists
  ifstream cons_pre( argv[2], ios::in );
  if( !cons_pre ){
    cerr << "error" << endl;
    exit( 1 );
  }

  ifstream out_pre( argv[1], ios::in );
  if( !out_pre ){
    cerr << "error" << endl;
    exit( 1 );
  }

  
  //Parser: read output from the previous criterion
  /*Output format of bsolo:
  SAT
c INFO: OSL Context initialized.
c Initial problem consists of 2 variables and 2 constraints.
c No problem reductions applied in OPT. instance.
c	preprocess terminated. Elapsed time: 0.005
c After prepocess the problem consists of 0 variables and 0 constraints.
c Initial Lower Bound: 2
c Sol. found (all assignments made)...
o 2
c NEW SOLUTION FOUND: 2 @ 0.005
s OPTIMUM FOUND
c Cost: 2
v x1 x2 
c Exit Code: 30
c Total time: 0.005 s
  UNSAT
c INFO: OSL Context initialized.
c Initial problem consists of 2 variables and 3 constraints.
s UNSATISFIABLE
c Exit Code: 20
c Total time: 0.011 s
  */
  bool b_optimal_founded = false;//indicate whether a optimal solution is founded
  bool b_value_founded = false;
  string line_buffer;
  string opt_value;
  int opt_holder;
  string::size_type position_index, position_index_2;
  char line_buffer_head;

  while(getline( out_pre, line_buffer )){
    line_buffer_head = *(line_buffer.begin());
    if( line_buffer_head != 's' ){
      if( line_buffer_head == 'o' ){
				position_index = line_buffer.find_first_not_of( 'o' );
				if( position_index != string::npos ){
					opt_value = line_buffer.substr( position_index );
					opt_holder = atoi( opt_value.c_str());
					b_optimal_founded = true; 
				}
      }
    }
    else{
      position_index = line_buffer.find( "OPTIMUM FOUND" );
      if( position_index != string::npos ){
        b_optimal_founded = true;
      }
    }
  }

  //Is there an optimal solution?
  if( !b_optimal_founded ){//no, exit
    ////
    //cout << "optimal not found "<< endl;
    ////
     return 2;
  }
  
  //What is the optimal value?
  // opt_value
  
  //Parser: extract the object function in previous criterion

  string new_constraint_buffer, new_constraint;
  //Get 1st line of the cons_pre
  while(getline( cons_pre, line_buffer ))
	{
		if( *line_buffer.begin() != '*')
			break;
	}
  //get rid of "min: " and ";"
  new_constraint_buffer = line_buffer.substr( 4 );
  position_index = new_constraint_buffer.size();
  new_constraint = new_constraint_buffer.substr( 0, position_index-1 );
  ///////
  //cout << "new_constraint: " << new_constraint << " <= " << opt_value << endl;  
  ///////



  //after checking that there is a optimal solution to previous criterion 
  //open output file for new criterion
  ofstream cons_val( argv[3], ios:: trunc );
  if( !cons_val ){
    exit( 1 );
  }
    ofstream cons_one_line( argv[4], ios:: trunc );
    if( !cons_one_line ){
      exit( 1 );
    }
    cons_one_line << new_constraint << " <=" << opt_holder << ";";
		cons_val << opt_holder;

  return 0;
}
