/* Generate opb file to PB solvers
 * argv[1]: the output opb file
 * argv[2]: a priority batch
 */

#include <iostream>
#include <fstream>
#include <string>
#include <set>
#include <map>
#include <algorithm>
#include <sstream>
#include <list>


using namespace std;


int main (int argc, char ** argv )
{
/*	if()//wrong number of input arguments
	{
		exit(1);
	}*/

	ofstream of_opb( argv[1], ios::trunc );
	if ( !of_opb  )
	{
	//open file for output failed
		exit ( 2 );
	}

	//open input file for one priority batch
	ifstream if_batch(argv[2], ios::in);
	if( !if_batch )
	{
		exit(3);
	}

	string str_buf;
	stringstream ss_buf;
	int i_vars; //numbers of variables
	string str_criteria_set; //file name of criteria set
	int i_constraint; // numbers of absolute criteria
	string str_constraint; // file name of constraint 

	map<int, int> map_r_cri;

	getline (if_batch, str_buf);
	ss_buf << str_buf;
	ss_buf >> i_vars;
	getline (if_batch, str_criteria_set);
	ss_buf.clear();
	ss_buf.str("");
	getline (if_batch, str_buf);
	ss_buf << str_buf;
	ss_buf >> i_constraint;
	getline( if_batch, str_constraint);

	if_batch.close();

	//open relative criteria set file
	ifstream if_criteria_set( str_criteria_set.c_str(), ios::in );
	if( !if_criteria_set )
	{
		exit (3);
	}
	string str_criterion;
	while( getline(if_criteria_set, str_criterion) )
	{
		//each relative criterion
		ifstream if_criterion(str_criterion.c_str(), ios::in);
		if( !if_criterion )
		{
			exit (4);
		}
		int i_relative_idx = 1;
		int i_coefficient;
		int i_weight;
		if_criterion >> i_weight;
		while( if_criterion >> i_coefficient )
		{
			map_r_cri[i_relative_idx] += i_weight * i_coefficient;
			i_relative_idx ++;
		}
			
		if_criterion.close();
	}
	if_criteria_set.close();
	
	//open absolute criteria file -- str_constraint
	ifstream if_constraint( str_constraint.c_str(), ios::in);
	if( !if_constraint )
	{
		exit(3);
	}

	string str_constraint_line;
	int i_constraint_idx = 0;
	int i_target; //rhs of a constraint
	stringstream ss_target;
	map <int, char> map_constraint_type;
	map <int, int> map_constraint_target;
	map <int, list<int> > map_constraint_coefficient;


  while(getline(if_constraint, str_constraint_line))
	{
		switch( i_constraint_idx %3)
		{
			case 0://type 
				 map_constraint_type[ i_constraint_idx / 3 ] = *str_constraint_line.begin();
				// cout << "type" << i_constraint_idx/3 << endl ;
				break;
			case 1://target
				// cout << "target" << i_constraint_idx/3 << endl ;
				ss_target.clear();
				ss_target.str("");
				ss_target << str_constraint_line;
				ss_target >> i_target;
				map_constraint_target[ i_constraint_idx / 3 ] = i_target;
				break;
			default://constraint
				 //cout << "cons" << i_constraint_idx/3 << endl ;
				stringstream ss_coefficient;
				string str_coefficient;
				int i_coefficient;
				size_t szt_beg = 0;
				size_t szt_end = 0;
				//szt_end = str_constraint_line.find_first_of( ' ', szt_beg );
				//while( szt_end != string::npos && szt_beg < str_constraint_line.size() )
				while( szt_beg != string::npos  )
				{
					szt_end = str_constraint_line.find_first_of( ' ', szt_beg );
					if(szt_end != string::npos)
						str_coefficient = str_constraint_line.substr( szt_beg, szt_end-szt_beg);
					else
						str_coefficient = str_constraint_line.substr( szt_beg);
					ss_coefficient.clear();
					ss_coefficient.str("");
					ss_coefficient << str_coefficient ;
					ss_coefficient >> i_coefficient; 
					map_constraint_coefficient[ i_constraint_idx / 3 ].push_back( i_coefficient );
					//szt_beg = szt_end + 1;
					if(szt_end != string::npos)
						szt_beg = str_constraint_line.find_first_not_of (' ', szt_end);
					else
						szt_beg = string::npos;
				}
				/*if (szt_beg < str_constraint_line.size())
				{
					str_coefficient = str_constraint_line;
					ss_coefficient.clear();
					ss_coefficient.str("");
					ss_coefficient << str_coefficient ;
					ss_coefficient >> i_coefficient; 
					map_constraint_coefficient[ i_constraint_idx / 3].push_back( i_coefficient );
				}*/
		}
		i_constraint_idx++;
	}
	//size_t szt_constraint = min( map_constraint_coefficient.size(), map_constraint_type.size() );
	//szt_constraint = min( map_constraint_target.size(), szt_constraint );
	of_opb << "* #variable= " << i_vars << " #constraint= " << i_constraint << endl;
	of_opb << "min: ";

	map<int, int>::iterator iter_var = map_r_cri.begin();
	while( iter_var != map_r_cri.end())
	{
		if( iter_var -> second > 0)
		{
			of_opb << "+" << iter_var->second << " x" << iter_var->first << ' ';
		}
		else if( iter_var->second < 0)
		{
			of_opb << iter_var->second << " x" << iter_var->first << ' ' ;
		}
		else
		{}
		iter_var++;
	}
	of_opb << ";" << endl;
	
	int  i_cons_idx = 0;
	while( i_cons_idx < i_constraint )
	{
		if( map_constraint_coefficient.find( i_cons_idx ) == map_constraint_coefficient.end() ||
				map_constraint_type.find( i_cons_idx ) == map_constraint_type.end() ||
				map_constraint_target.find( i_cons_idx ) == map_constraint_target.end()
			)
		{
			cout << map_constraint_coefficient.begin()->first << ' ' <<
              map_constraint_type.begin()->first << ' ' <<
              map_constraint_target.begin()->first << endl;
			i_cons_idx++;
			continue;
		}
		int i_var_idx = 1;
		list<int>  ::iterator iter_coeff_list = map_constraint_coefficient.find( i_cons_idx )->second.begin();
		while( iter_coeff_list != map_constraint_coefficient.find( i_cons_idx )->second.end() )
		{
			if( *iter_coeff_list > 0 )
			{
				of_opb << '+' << *iter_coeff_list << " x" << i_var_idx << ' ' ;
			}
			else if (*iter_coeff_list < 0 )
			{
				of_opb << *iter_coeff_list << " x" << i_var_idx << ' ' ;
			}
			else
			{}
			i_var_idx ++;
			iter_coeff_list++;
		}
		switch(map_constraint_type[i_cons_idx])
		{
			case 'a':
				of_opb << "> ";
				break;
			case 'b':
				of_opb << ">= ";
				break;
			case 'c':
				of_opb << "= ";
				break;
			case 'd':
				of_opb << "<= ";
				break;
			case 'e':
				of_opb << "< ";
				break;
			default:
				break;
		}
		of_opb << map_constraint_target[i_cons_idx] << ";" << endl;
		i_cons_idx ++;
	}

	return 0;
}
