//Build next PB constraints based on the previous output
//(For prioritization of minimization criteria)

#include <iostream>
#include <fstream>
#include <cstdio>
#include <string>
#include <cstdlib>


using namespace std;

//argv[0]	COMMAND
//argv[2]	input file name: constraints file (minisat+ opb format) of previous criterion
//argv[1]	input file name: output of PB solvers to previous criterion
//argv[3]	output file name: the evaluated opt value
//argv[4]	output file name: the newly added constraint (only one constraint)

int main( int argc, char *argv[] ){

  //check whether input files exists
  ifstream cons_pre( argv[2], ios::in );
  if( !cons_pre ){
    cerr << "error" << endl;
    exit( 1 );
  }

  ifstream out_pre( argv[1], ios::in );
  if( !out_pre ){
    cerr << "error" << endl;
    exit( 1 );
  }

/*  ifstream cons_next( argv[3], ios::in );
  if( !cons_next ){
    cerr << "error" << endl;
    exit( 1 );
  }*/

  
  //Parser: read output from the previous criterion
  /*Output format of opbdp:
  SAT
opbdp 1.1 #2  Copyright (C) 1995-2005 Peter Barth <barth@mpi-sb.mpg.de>
              Max-Planck-Institut f. Informatik
type opbdp -H for help
Tautology deleted in line 2 
Tautology deleted in line 3 
2 equations changed into 4 inequalities 
problem consists of 2 inequalities and 2 variables 
Fixing : x1 x2 
2 Literals have been fixed by 'fixing' 
Preprocessingtime (secs) : 0.00
                               Value        Nodes /    Fixed /       Time 
Global Minimum: ****** 2 ******
Explored Nodes: 0   Max. Depth: 0   Fixed Literals: 0
Time (secs) : init(0.00) prepro(0.00) solve(0.00)

  UNSAT
opbdp 1.1 #2  Copyright (C) 1995-2005 Peter Barth <barth@mpi-sb.mpg.de>
              Max-Planck-Institut f. Informatik
type opbdp -H for help
Tautology deleted in line 2 
Tautology deleted in line 3 
Tautology deleted in line 4 
3 equations changed into 6 inequalities 
problem consists of 3 inequalities and 2 variables 
Fixing : x1 
'fixing' detected inconsistency
Preprocessingtime (secs) : 0.00
                               Value        Nodes /    Fixed /       Time 
Constraint Set is unsatisfiable
Explored Nodes: 0   Max. Depth: 0   Fixed Literals: 0
Time (secs) : init(0.00) prepro(0.00) solve(0.00)


KEYWORD "Global"
  */
  bool b_optimal_founded = false;//indicate whether a optimal solution is founded
  bool b_value_founded = false;
  string line_buffer;
  string opt_value;
  string::size_type opt_length_holder;
  string::size_type position_index_1, position_index_2, position_index_3;
  char line_buffer_head;
  int opt_holder;

  while(getline( out_pre, line_buffer )){
    position_index_1 = line_buffer.find( "Global" );
    if( position_index_1 != string::npos ){
      b_optimal_founded = true;
      position_index_1 = line_buffer.find( "******" );
      if( position_index_1 != string::npos ){
        position_index_2 = line_buffer.find_first_not_of( "*", position_index_1 );
        if( position_index_2 != string::npos ){
	  opt_value = line_buffer.substr( position_index_2 );
	  opt_holder = atoi( opt_value.c_str());
	}
      }
    }
    else{
      position_index_1 = line_buffer.find( "New Local Minimum:" );
      if( position_index_1 != string::npos ){
        b_optimal_founded = true;
	position_index_2 = line_buffer.find_first_not_of( "New Local Minimum:", position_index_1 );
        if( position_index_2 != string::npos ){
	  opt_value = line_buffer.substr( position_index_2 );
	  opt_holder = atoi( opt_value.c_str());
	}
      }
    }
  }

  //Is there an optimal solution?
  if( !b_optimal_founded || opt_value.empty() ){//no, exit
    ////
    //cout << "optimal not found "<< endl;
    ////
     return 3;
  }
  
  //What is the optimal value?
  // opt_value
  
  //Parser: extract the object function in previous criterion

  string new_constraint_buffer, new_constraint;
  //Get 1st line of the cons_pre
  while(getline( cons_pre, line_buffer ))
	{
		if( *line_buffer.begin() != '*')
			break;
	}
  //get rid of "min: " and ";"
  new_constraint_buffer = line_buffer.substr( 4 );
  position_index_1 = new_constraint_buffer.size();
  new_constraint = new_constraint_buffer.substr( 0, position_index_1-1 );
  ///////
  //cout << "new_constraint: " << new_constraint << " <= " << opt_value << endl;  
  ///////



  //after checking that there is a optimal solution to previous criterion 
  //open output file for new criterion
  ofstream cons_val( argv[3], ios:: trunc );
  if( !cons_val ){
    exit( 1 );
  }
  
    ofstream cons_one_line( argv[4], ios:: trunc );
    if( !cons_one_line ){
      exit( 1 );
    }
    cons_one_line << new_constraint << " <=" << opt_holder << ";";
		cons_val << opt_holder;

  return 0;
}
